use strict;
use warnings;
use Test::More tests => 1;
use_ok('POE::Component::IRC::Plugin::QueryDNSBL');
diag("Testing POE::Component::IRC::Plugin::QueryDNSBL $POE::Component::IRC::Plugin::QueryDNSBL::VERSION, Perl $], $^X" );
